export module person;

import <string>;
import <format>;
import <compare>;
import <iostream>;
import <iomanip>;

export class Person
{
public:
	// Two-parameter constructor automatically creates initials and
	// delegates the work to the three-parameter constructor.
	Person(std::string firstName, std::string lastName)
		: Person{ std::move(firstName), std::move(lastName),
		std::format("{}{}", firstName[0], lastName[0]) }
	{
	}

	Person() = default;

	Person(std::string firstName, std::string lastName, std::string initials)
		: m_firstName{ std::move(firstName) }
		, m_lastName{ std::move(lastName) }
		, m_initials{ std::move(initials) }
	{
	}

	const std::string& getFirstName() const { return m_firstName; }
	void setFirstName(std::string firstName) { m_firstName = std::move(firstName); }

	const std::string& getLastName() const { return m_lastName; }
	void setLastName(std::string lastName) { m_lastName = std::move(lastName); }

	const std::string& getInitials() const { return m_initials; }
	void setInitials(std::string initials) { m_initials = std::move(initials); }

	void output(std::ostream& output) const
	{
		output << std::format("{} {} ({})",
			getFirstName(), getLastName(), getInitials()) << std::endl;
	}

	// Only this single line of code is needed to add support
	// for all six comparison operators.
	[[nodiscard]] auto operator<=>(const Person&) const = default;

private:
	std::string m_firstName;
	std::string m_lastName;
	std::string m_initials;
};

export std::ostream& operator<<(std::ostream& os, const Person& person)
{
	os << std::quoted(person.getFirstName())
		<< std::quoted(person.getLastName())
		<< std::quoted(person.getInitials());

	return os;
}

export std::istream& operator>>(std::istream& is, Person& person)
{
	std::string firstName, lastName, initials;
	is >> std::quoted(firstName)
		>> std::quoted(lastName)
		>> std::quoted(initials);

	person.setFirstName(std::move(firstName));
	person.setLastName(std::move(lastName));
	person.setInitials(std::move(initials));

	return is;
}
